@@ -2,11 +2,11 @@ import builtins
 import os
 import sys
 from array import array
-from collections import Counter, defaultdict, deque
+from collections import Counter, defaultdict, deque, UserDict, UserList
 from dataclasses import dataclass, fields, is_dataclass
-import inspect
 from itertools import islice
 from typing import (
+    DefaultDict,
     TYPE_CHECKING,
     Any,
     Callable,
@@ -18,16 +18,32 @@ from typing import (
     Union,
     Tuple,
 )
+from types import MappingProxyType
 
-from rich.highlighter import ReprHighlighter
+try:
+    import attr as _attr_module
+except ImportError:  # pragma: no cover
+    _attr_module = None  # type: ignore
 
+
+def _is_attr_object(obj: Any) -> bool:
+    """Check if an object was created with attrs module."""
+    return _attr_module is not None and _attr_module.has(type(obj))
+
+
+def _get_attr_fields(obj: Any) -> Iterable["_attr_module.Attribute[Any]"]:
+    """Get fields for an attrs object."""
+    return _attr_module.fields(type(obj)) if _attr_module is not None else []
+
+
+from .highlighter import ReprHighlighter
 from . import get_console
 from ._loop import loop_last
 from ._pick import pick_bool
 from .abc import RichRenderable
 from .cells import cell_len
 from .highlighter import ReprHighlighter
-from .jupyter import JupyterRenderable
+from .jupyter import JupyterMixin, JupyterRenderable
 from .measure import Measurement
 from .text import Text
 
@@ -43,12 +59,12 @@ if TYPE_CHECKING:
 
 
 def install(
-    console: "Console" = None,
+    console: Optional["Console"] = None,
     overflow: "OverflowMethod" = "ignore",
     crop: bool = False,
     indent_guides: bool = False,
-    max_length: int = None,
-    max_string: int = None,
+    max_length: Optional[int] = None,
+    max_string: Optional[int] = None,
     expand_all: bool = False,
 ) -> None:
     """Install automatic pretty printing in the Python REPL.
@@ -99,6 +115,9 @@ def install(
         if console.is_jupyter and any(attr.startswith("_repr_") for attr in dir(value)):
             return
 
+        if hasattr(value, "_repr_mimebundle_"):
+            return
+
         # certain renderables should start on a new line
         if isinstance(value, ConsoleRenderable):
             console.line()
@@ -130,7 +149,7 @@ def install(
         sys.displayhook = display_hook
 
 
-class Pretty:
+class Pretty(JupyterMixin):
     """A rich renderable that pretty prints an object.
 
     Args:
@@ -152,15 +171,15 @@ class Pretty:
     def __init__(
         self,
         _object: Any,
-        highlighter: "HighlighterType" = None,
+        highlighter: Optional["HighlighterType"] = None,
         *,
         indent_size: int = 4,
-        justify: "JustifyMethod" = None,
+        justify: Optional["JustifyMethod"] = None,
         overflow: Optional["OverflowMethod"] = None,
         no_wrap: Optional[bool] = False,
         indent_guides: bool = False,
-        max_length: int = None,
-        max_string: int = None,
+        max_length: Optional[int] = None,
+        max_string: Optional[int] = None,
         expand_all: bool = False,
         margin: int = 0,
         insert_line: bool = False,
@@ -228,7 +247,7 @@ class Pretty:
         return Measurement(text_width, text_width)
 
 
-def _get_braces_for_defaultdict(_object: defaultdict) -> Tuple[str, str, str]:
+def _get_braces_for_defaultdict(_object: DefaultDict[Any, Any]) -> Tuple[str, str, str]:
     return (
         f"defaultdict({_object.default_factory!r}, {{",
         "})",
@@ -236,7 +255,7 @@ def _get_braces_for_defaultdict(_object: defaultdict) -> Tuple[str, str, str]:
     )
 
 
-def _get_braces_for_array(_object: array) -> Tuple[str, str, str]:
+def _get_braces_for_array(_object: "array[Any]") -> Tuple[str, str, str]:
     return (f"array({_object.typecode!r}, [", "])", "array({_object.typecode!r})")
 
 
@@ -247,13 +266,16 @@ _BRACES: Dict[type, Callable[[Any], Tuple[str, str, str]]] = {
     Counter: lambda _object: ("Counter({", "})", "Counter()"),
     deque: lambda _object: ("deque([", "])", "deque()"),
     dict: lambda _object: ("{", "}", "{}"),
+    UserDict: lambda _object: ("{", "}", "{}"),
     frozenset: lambda _object: ("frozenset({", "})", "frozenset()"),
     list: lambda _object: ("[", "]", "[]"),
+    UserList: lambda _object: ("[", "]", "[]"),
     set: lambda _object: ("{", "}", "set()"),
     tuple: lambda _object: ("(", ")", "()"),
+    MappingProxyType: lambda _object: ("mappingproxy({", "})", "mappingproxy({})"),
 }
 _CONTAINERS = tuple(_BRACES.keys())
-_MAPPING_CONTAINERS = (dict, os._Environ)
+_MAPPING_CONTAINERS = (dict, os._Environ, MappingProxyType, UserDict)
 
 
 def is_expandable(obj: Any) -> bool:
@@ -262,6 +284,7 @@ def is_expandable(obj: Any) -> bool:
         isinstance(obj, _CONTAINERS)
         or (is_dataclass(obj) and not isinstance(obj, type))
         or hasattr(obj, "__rich_repr__")
+        or _is_attr_object(obj)
     )
 
 
@@ -411,7 +434,9 @@ class _Line:
         return f"{self.whitespace}{self.text}{self.node or ''}{self.suffix}"
 
 
-def traverse(_object: Any, max_length: int = None, max_string: int = None) -> Node:
+def traverse(
+    _object: Any, max_length: Optional[int] = None, max_string: Optional[int] = None
+) -> Node:
     """Traverse object and generate a tree.
 
     Args:
@@ -451,7 +476,7 @@ def traverse(_object: Any, max_length: int = None, max_string: int = None) -> No
         py_version = (sys.version_info.major, sys.version_info.minor)
         children: List[Node]
 
-        def iter_rich_args(rich_args) -> Iterable[Union[Any, Tuple[str, Any]]]:
+        def iter_rich_args(rich_args: Any) -> Iterable[Union[Any, Tuple[str, Any]]]:
             for arg in rich_args:
                 if isinstance(arg, tuple):
                     if len(arg) == 3:
@@ -485,7 +510,6 @@ def traverse(_object: Any, max_length: int = None, max_string: int = None) -> No
                         child_node = _traverse(child)
                         child_node.last = last
                         child_node.key_repr = key
-                        child_node.last = last
                         child_node.key_separator = "="
                         append(child_node)
                     else:
@@ -496,6 +520,50 @@ def traverse(_object: Any, max_length: int = None, max_string: int = None) -> No
                 node = Node(
                     value_repr=f"{obj.__class__.__name__}()", children=[], last=root
                 )
+        elif _is_attr_object(obj):
+            children = []
+            append = children.append
+
+            attr_fields = _get_attr_fields(obj)
+            if attr_fields:
+                node = Node(
+                    open_brace=f"{obj.__class__.__name__}(",
+                    close_brace=")",
+                    children=children,
+                    last=root,
+                )
+
+                def iter_attrs() -> Iterable[
+                    Tuple[str, Any, Optional[Callable[[Any], str]]]
+                ]:
+                    """Iterate over attr fields and values."""
+                    for attr in attr_fields:
+                        if attr.repr:
+                            try:
+                                value = getattr(obj, attr.name)
+                            except Exception as error:
+                                # Can happen, albeit rarely
+                                yield (attr.name, error, None)
+                            else:
+                                yield (
+                                    attr.name,
+                                    value,
+                                    attr.repr if callable(attr.repr) else None,
+                                )
+
+                for last, (name, value, repr_callable) in loop_last(iter_attrs()):
+                    if repr_callable:
+                        child_node = Node(value_repr=str(repr_callable(value)))
+                    else:
+                        child_node = _traverse(value)
+                    child_node.last = last
+                    child_node.key_repr = name
+                    child_node.key_separator = "="
+                    append(child_node)
+            else:
+                node = Node(
+                    value_repr=f"{obj.__class__.__name__}()", children=[], last=root
+                )
 
         elif (
             is_dataclass(obj)
@@ -529,7 +597,12 @@ def traverse(_object: Any, max_length: int = None, max_string: int = None) -> No
 
             pop_visited(obj_id)
 
-        elif obj_type in _CONTAINERS:
+        elif isinstance(obj, _CONTAINERS):
+            for container_type in _CONTAINERS:
+                if isinstance(obj, container_type):
+                    obj_type = container_type
+                    break
+
             obj_id = id(obj)
             if obj_id in visited_ids:
                 # Recursion detected
@@ -538,7 +611,9 @@ def traverse(_object: Any, max_length: int = None, max_string: int = None) -> No
 
             open_brace, close_brace, empty = _BRACES[obj_type](obj)
 
-            if obj:
+            if obj_type.__repr__ != type(obj).__repr__:
+                node = Node(value_repr=to_repr(obj), last=root)
+            elif obj:
                 children = []
                 node = Node(
                     open_brace=open_brace,
@@ -587,8 +662,8 @@ def pretty_repr(
     *,
     max_width: int = 80,
     indent_size: int = 4,
-    max_length: int = None,
-    max_string: int = None,
+    max_length: Optional[int] = None,
+    max_string: Optional[int] = None,
     expand_all: bool = False,
 ) -> str:
     """Prettify repr string by expanding on to new lines to fit within a given width.
@@ -620,12 +695,12 @@ def pretty_repr(
 def pprint(
     _object: Any,
     *,
-    console: "Console" = None,
+    console: Optional["Console"] = None,
     indent_guides: bool = True,
-    max_length: int = None,
-    max_string: int = None,
+    max_length: Optional[int] = None,
+    max_string: Optional[int] = None,
     expand_all: bool = False,
-):
+) -> None:
     """A convenience function for pretty printing.
 
     Args:
@@ -654,8 +729,9 @@ def pprint(
 if __name__ == "__main__":  # pragma: no cover
 
     class BrokenRepr:
-        def __repr__(self):
+        def __repr__(self) -> str:
             1 / 0
+            return "this will fail"
 
     d = defaultdict(int)
     d["foo"] = 5
