@@ -5,4 +5,8 @@ from .abc import RichRenderable
 
 def is_renderable(check_object: Any) -> bool:
     """Check if an object may be rendered by Rich."""
-    return isinstance(check_object, str) or isinstance(check_object, RichRenderable)
+    return (
+        isinstance(check_object, str)
+        or hasattr(check_object, "__rich__")
+        or hasattr(check_object, "__rich_console__")
+    )
